# schemas.py
POSITION_COLUMNS = [
    'position',
    'num_of_variants',
    'histogram',
    'enhancing_score',
    'neutral_score',
    'rheostat_score',
    'weighted_rheostat_score',
    'toggle_score',
    'binary',
    'average',
    'st_dev',
    'assignment',
]

